﻿using System.Collections;
using Holoville.HOTween;
using Holoville.HOTween.Path;
using UnityEngine;

public class GameManager : MonoBehaviour {

	#region SINGLETON

	private static GameManager _instance;

	public static GameManager Instance {
		get { return _instance; }
	}

	public void Awake() {
		if (GameManager.Instance != null) {
			DestroyImmediate(gameObject);
			return;
		}
		_instance = this;
	}

	public void OnApplicationQuit() {
		_instance = null;
	}

	#endregion

	public HOPath cameraPath;
	public AudioSource music;
	public bool skipIntro;
	public CameraMode cameraMode;
	public Transform survivorSavedParticles;
	public Transform foodStartPoint;
	public GameObject player;
	public SurvivorSpawns spawns;
	public Survivor ladderSurvivor;

	public float TimeScaleControl {
		set { Time.timeScale = value; }
		get { return Time.timeScale; }
	}

	protected IEnumerator Start() {
		yield return new WaitForEndOfFrame();

		if (!skipIntro) {
			player.SetActive(false);	
			Time.timeScale = 0.05f;
			music.time = 9f;
			music.volume = 0f;
			music.Play();
			HOTween.To(music, 2f, "volume", 1f);
			HOTween.To(Camera.main.transform, 60f,
			           new TweenParms().Prop("position",
			                                 cameraPath.MakePlugVector3Path().OrientToPath(true, 0.2f, Axis.Z)).Loops(
			                                 	-1).Ease(
			                                 		EaseType.Linear).UpdateType(UpdateType.TimeScaleIndependentUpdate));
			UIManager.Instance.SwitchView(UIManager.View.Title, true);
			UIManager.Instance.FadeBlackout(0f, 6f);

		} else {
			UIManager.Instance.FadeBlackout(0f, 0f);
			Time.timeScale = 1f;
			cameraMode.ToggleInGame(true);
			UIManager.Instance.SwitchView(UIManager.View.InGame, true);
			spawns.enabled = true;
			UIManager.Instance.AddPersonMarker(ladderSurvivor);
		}
	}

	protected void Update() {
		switch (UIManager.Instance.currentView) {
			case UIManager.View.Title:
				if (Input.GetButtonDown("Action")) {
					UIManager.Instance.FadeBlackout(1f);
					UIManager.Instance.SwitchView(UIManager.View.Story);
				}
				break;
			case UIManager.View.Story:
				if (Input.GetButtonDown("Action")) {
					UIManager.Instance.SwitchView(UIManager.View.InGame);
					UIManager.Instance.FadeBlackout(0f);
					HOTween.Kill(Camera.main.transform);
					cameraMode.ToggleInGame(true);
					player.SetActive(true);
					Time.timeScale = 1f;
					spawns.enabled = true;
					UIManager.Instance.AddPersonMarker(ladderSurvivor);
//					HOTween.To(this, 2f,
//					           new TweenParms().Prop("TimeScaleControl", 1f).Delay(1f).UpdateType(
//					           	UpdateType.TimeScaleIndependentUpdate));
				}
				break;
			case UIManager.View.GameOver:
				if (Input.GetKeyDown(KeyCode.R)) {
					player.SetActive(false);
					Application.LoadLevel(Application.loadedLevel);
				}
				break;
		}
	}

}