﻿using System.Collections;
using Holoville.HOTween;
using UnityEngine;

public class Player : MonoBehaviour {

	// above zero is human, below zero is zombie between -1 and 1
	public float humanLevel;

	// whilst zombie this increases over time.
	public float feedingNeed;
	public float feedingIncrease;

	public LayerMask distractionLayerMask;
	public float distractionRadius;

	private Transform myTransform;
	private Animator animator;
	public SkinnedMeshRenderer skinRenderer;
	public Material skinMaterial;
	public LayerMask aidItemsLayerMask;

	// audio components
	public AdjustGain gain;
	public AudioDistortionFilter distortion;

	public HoldObject holding;
	private bool distracting;
	public float handOverrideWeight;
	public Transform holdingObjectParent;
	public Transform ikRight;
	public Transform ikLeft;
	public bool enableIK;
	public CameraFX cameraFx;
	private float deathTimer;
	private const float deathTimeout = 5f;
	private bool dead;
	public CameraFollow camFollow;

	public float HumanLevel01 {
		get {
			return Mathf.Clamp01(humanLevel * 0.5f + 0.5f);
		}
	}

	protected void Start() {
		myTransform = transform;
		animator = GetComponent<Animator>();
	}

	protected void Update() {
		humanLevel = Mathf.Clamp(humanLevel, -1f, 1f);
		distortion.distortionLevel = Mathf.Clamp(((1 - HumanLevel01) - 0.4f) * 1.5f, 0f, 0.95f);
		gain.gain = 0.8f - (1 - HumanLevel01) * 0.7f;
		camFollow.height = 4 + HumanLevel01 * 10;
		camFollow.distance = 6 + 2 * HumanLevel01;
		skinRenderer.SetBlendShapeWeight(0, (1 - HumanLevel01) * 100f);
		skinMaterial.SetFloat("_Z", 1 - HumanLevel01);
		float nearestDistance = float.MaxValue;
		if (holding != null) {
			UIManager.Instance.SpaceBarDoes("DROP " + holding.name.ToUpper());
			if (Input.GetButtonDown("Action")) {
				if (holding.tag == "Ladder") {
					foreach (Collider c in Physics.OverlapSphere(myTransform.position, 0.5f)) {
						if (c.tag == "LadderDrop") {
							c.GetComponent<LadderDrop>().DoLadder();
							Destroy(holding.gameObject);
							enableIK = false;
							holding = null;
							SurvivorSpawns ss = (SurvivorSpawns)FindObjectOfType(typeof(SurvivorSpawns));
							humanLevel += 0.2f;
							return;
						}
					}
				}

				RaycastHit hitInfo;
				if (Physics.Raycast(myTransform.position + myTransform.forward * 0.5f + Vector3.up, Vector3.down, out hitInfo,
								10f)) {
					holding.transform.parent = null;
					holding.isBeingHeld = false;
					holding.transform.position = hitInfo.point;
					enableIK = false;
					holding = null;
				}
			}
		} else {
			enableIK = false;
			Collider nearestCollider = null;
			foreach (Collider c in Physics.OverlapSphere(myTransform.position, 0.5f, aidItemsLayerMask)) {
				float dist = (c.transform.position - myTransform.position).sqrMagnitude;
				if (dist < nearestDistance) {
					nearestDistance = dist;
					nearestCollider = c;
				}
			}

			if (nearestCollider != null) {
				switch (nearestCollider.tag) {
					case "Food":
					case "FirstAid":
					case "Ladder":
						UIManager.Instance.SpaceBarDoes("PICK UP " + nearestCollider.gameObject.name.ToUpper());
						if (Input.GetButtonDown("Action")) {
							holding = nearestCollider.GetComponent<HoldObject>();
							holding.isBeingHeld = true;
							enableIK = true;
							ikLeft = holding.ikLeftTarget;
							ikRight = holding.ikRightTarget;
							nearestCollider.transform.parent = holdingObjectParent;
							nearestCollider.transform.localPosition = Vector3.zero;
							nearestCollider.transform.localRotation = Quaternion.identity;
						}
						break;
				}
			} else {
				UIManager.Instance.SpaceBarDoes("CAUSE DISTRACTION");
				if (Input.GetButtonDown("Action")) {
					if (distracting) return;
					StartCoroutine(DoDistraction());
				}
			}
		}

		if (humanLevel < -0.5f) {
			feedingNeed += feedingIncrease * Time.deltaTime;
		} else if (humanLevel > 0.5f) {
			feedingNeed -= feedingIncrease * Time.deltaTime;
		}
		if (humanLevel <= -1f) {
			deathTimer += Time.deltaTime;
			if (deathTimer > deathTimeout && !dead) {
				dead = true;
				UIManager.Instance.GameOver("THE INFECTION OVERCAME YOU. WELCOME TO YOUR ETERNAL DEATH.");
			}
		} else {
			deathTimer = 0;
		}
		feedingNeed = Mathf.Clamp01(feedingNeed);
		animator.SetFloat("humanLevel", HumanLevel01);
		animator.SetLayerWeight(1, handOverrideWeight);
	}

	private void OnAnimatorIK() {
		if (enableIK) {
			animator.SetIKPositionWeight(AvatarIKGoal.LeftHand, 1f);
			animator.SetIKPositionWeight(AvatarIKGoal.RightHand, 1f);
			animator.SetIKRotationWeight(AvatarIKGoal.LeftHand, 1f);
			animator.SetIKRotationWeight(AvatarIKGoal.RightHand, 1f);
			if (ikLeft) {
				animator.SetIKPosition(AvatarIKGoal.LeftHand, ikLeft.position);
				animator.SetIKRotation(AvatarIKGoal.LeftHand, ikLeft.rotation);
			}
			if (ikRight) {
				animator.SetIKPosition(AvatarIKGoal.RightHand, ikRight.position);
				animator.SetIKRotation(AvatarIKGoal.RightHand, ikRight.rotation);
			}
		} else {
			animator.SetIKPositionWeight(AvatarIKGoal.RightHand, 0f);
			animator.SetIKRotationWeight(AvatarIKGoal.RightHand, 0f);
			animator.SetIKPositionWeight(AvatarIKGoal.LeftHand, 0f);
			animator.SetIKRotationWeight(AvatarIKGoal.LeftHand, 0f);
		}
	}


	private void OnControllerColliderHit(ControllerColliderHit hit) {
		Debug.Log("Hit : " + hit.collider);
		if (hit.collider.tag == "Zombie") {
			humanLevel -= 0.005f;
			cameraFx.FlashBad();
		}
	}

	private IEnumerator DoDistraction() {
		distracting = true;
		animator.Play("Distract", 1, 0f);
		HOTween.To(this, 0.5f, "handOverrideWeight", 1f);
		foreach (Collider zombie in Physics.OverlapSphere(myTransform.position, distractionRadius, distractionLayerMask)) {
			zombie.SendMessage("Distract", SendMessageOptions.DontRequireReceiver);
		}
		yield return new WaitForSeconds(1f);
		HOTween.To(this, 0.5f, "handOverrideWeight", 0f);
		yield return new WaitForSeconds(0.5f);
		distracting = false;
	}

	public void HumanSaved() {
		humanLevel += 0.1f;
		cameraFx.FlashGood();
	}
}