﻿using System.Collections;
using Holoville.HOTween;
using Holoville.HOTween.Plugins;
using UnityEngine;

public class Survivor : MonoBehaviour {

	public float runSpeed;
	public int spawnIndex;
	public SurvivorSpawns spawner;

	public enum State {
		NeedFood, NeedAid, Trapped, LostFriend, Trapped2ndFloor, Saved
	}

	private bool movingToTarget;
	public State state;
	public Transform target;
	public Survivor lostFriend;
	public LayerMask aidObjectsLayerMask;

	public float foodRadius;
	public float firstAidRadius;

	private NavMeshAgent agent;
	private Transform myTransform;


	private bool isSaved;
	public Animator animator;
	private Renderer myRenderer;
	private Player player;
	private CameraFX cameraFx;
	public bool isCover;

	protected void Start() {
		cameraFx = (CameraFX)FindObjectOfType(typeof(CameraFX));
		player = (Player)FindObjectOfType(typeof(Player));
		animator = GetComponent<Animator>();
		agent = GetComponent<NavMeshAgent>();
		myTransform = transform;
		myRenderer = GetComponentInChildren<Renderer>();
		// stay where u are...
		if (agent) {
			state = (State) Random.Range(0, 2);
			agent.speed = 0f;
			if (isCover) {
				animator.Play("cover");
			} else {
				animator.Play("against wall");
			}
		} else {
			animator.Play("wave");
		}
		StartCoroutine(StatusCheck());
	}

	private void OnTriggerEnter(Collider other) {
		switch (state) {
			case State.NeedFood:
				if (other.tag == "Food") {
					if (player.holding == other.GetComponent<HoldObject>()) {
						player.holding = null;
						player.enableIK = false;
					}
					other.transform.parent = null;
					other.transform.position = GameManager.Instance.foodStartPoint.position;
					other.transform.rotation = GameManager.Instance.foodStartPoint.rotation;
					player.HumanSaved();
					StartCoroutine(FadeOut());
				}
				break;
			case State.NeedAid:
				if (other.tag == "FirstAid") {
					Destroy(other.gameObject);
					if (player.holding == other.GetComponent<HoldObject>()) {
						player.holding = null;
						player.enableIK = false;
					}
					player.HumanSaved();
					StartCoroutine(FadeOut());
				}
				break;
		}
		if (other.tag == "Zombie" && movingToTarget) {
			cameraFx.FlashBad();

			RemoveFromSpawner();
			Destroy(gameObject);
			// TODO Polish this. Add Death animation and zombie attack animation
		}
	}

	private void RemoveFromSpawner() {
		if (isCover) {
			spawner.coversTaken[spawnIndex] = false;
		} else {
			spawner.sittingTaken[spawnIndex] = false;
		}
		spawner.numberOut--;
	}

	private IEnumerator StatusCheck() {
		yield return new WaitForSeconds(Random.value);
		while (!isSaved) {
			Debug.Log("Running status check");
			yield return new WaitForSeconds(2.5f);
			switch (state) {
				case State.NeedFood:
					foreach (Collider c in Physics.OverlapSphere(myTransform.position, foodRadius, aidObjectsLayerMask)) {
						if (c.tag == "Food") {
							if (c.GetComponent<HoldObject>().isBeingHeld) break;
							movingToTarget = true;

							agent.destination = c.transform.position;
							animator.SetBool("moving", true);
							// TODO Get up and run animation, and say thanks zombie!!
							isSaved = false;
							break;
						}
					}
					break;
				case State.NeedAid:
					foreach (Collider c in Physics.OverlapSphere(myTransform.position, firstAidRadius, aidObjectsLayerMask)) {
						if (c.tag == "FirstAid") {
							if (c.GetComponent<HoldObject>().isBeingHeld) break;
							movingToTarget = true;
							agent.destination = c.transform.position;

							animator.SetBool("moving", true);
							// TODO Get up and run animation, and say thanks zombie!!
							isSaved = false;
							break;
						}
					}
					break;
			}
		}
	}

	private void Update() {
		if (agent) {
			if (movingToTarget && animator.GetCurrentAnimatorStateInfo(0).IsName("Locomotion")) {
				agent.speed = runSpeed;
			}
			animator.SetFloat("speed", agent.speed);
			if (movingToTarget) {
				if (agent.remainingDistance < 0.1f) {
					agent.Stop();
					movingToTarget = false;
				}
			}
		}
	}

	public IEnumerator FadeOut() {
		animator.SetTrigger("saved");
		yield return new WaitForSeconds(1f);
		GameManager.Instance.survivorSavedParticles.position = myTransform.position + Vector3.up * 1f;
		float currentCutOff = 0f;
		while (currentCutOff < 1f) {
			myRenderer.material.SetFloat("_Cutoff", currentCutOff);
			currentCutOff += Time.deltaTime * 0.5f;
			yield return new WaitForEndOfFrame();
		}
		GameManager.Instance.survivorSavedParticles.position = Vector3.up * 1000f;
		if (agent) RemoveFromSpawner();
		Destroy(gameObject);
	}
}
