﻿using UnityEngine;
using System.Collections;

public class Zombie : MonoBehaviour {

	public enum State {
		Idle, DistractedMove, Chase,
		Ramble
	}

	public State state;

	private Transform target;
	private Player targetPlayer;

	public LayerMask losLayers;

	public float minKarmaLevelForChase;
	public float maxKarmaLevelForChase;
	public float minMoveSpeed;
	public float maxMoveSpeed;
	public float minShambleSpeed;
	public float maxShambleSpeed;
	public float animationBlendSmoothing;

	// components
	private NavMeshAgent agent;
	private Transform myTransform;

	// data
	private float chaseKarmaLevel;
	private bool hasLos;
	private float moveSpeed;
	private Vector3 distractionLocation;
	private Animator animator;
	private float shambleSpeed;
	private float animationSpeed;
	private bool readyForRandom;
	public LayerMask randomCheckLayerMask;

	// Use this for initialization
	void Start() {
		animator = GetComponent<Animator>();
		//animator.Play("Blend Tree", 0, Random.value);
		targetPlayer = (Player)FindObjectOfType(typeof(Player));
		target = targetPlayer.transform;
		myTransform = transform;
		state = State.Idle;
		chaseKarmaLevel = Random.Range(minKarmaLevelForChase, maxKarmaLevelForChase);
		agent = GetComponent<NavMeshAgent>();
		moveSpeed = Random.Range(minMoveSpeed, maxMoveSpeed);
		shambleSpeed = Random.Range(minShambleSpeed, maxShambleSpeed);
		StartCoroutine(LOSCheck());
		StartCoroutine(RandomCheck());
	}

	private IEnumerator RandomCheck() {
		yield return new WaitForSeconds(Random.Range(1f, 3f));
		while (true) {
			if (readyForRandom) {
				readyForRandom = false;
				state = State.Ramble;
				RaycastHit hitInfo;
				Physics.Raycast(
					myTransform.position + new Vector3(Random.Range(-4f, 4f), 0, Random.Range(-4f, 4f)) + Vector3.up * 10f,
					Vector3.down, out hitInfo, 20f, randomCheckLayerMask);
				agent.destination = hitInfo.point;
			}
			yield return new WaitForSeconds(Random.Range(1f, 4f));
		}
	}

	private IEnumerator LOSCheck() {
		yield return new WaitForSeconds(Random.Range(1f, 5f));
		while (true) {
			yield return new WaitForSeconds(5f);
			hasLos = !Physics.Linecast(myTransform.position+Vector3.up*1.5f, target.position + Vector3.up*1.5f, losLayers);
		}
	}


	private void Distract() {
		if (hasLos) {
			state = State.DistractedMove;
			distractionLocation = target.position;
		}
	}

	// Update is called once per frame
	void Update() {

		if (hasLos && targetPlayer.humanLevel > chaseKarmaLevel) {
			state = State.Chase;
		}
		if (state == State.Chase && targetPlayer.humanLevel < chaseKarmaLevel) {
			distractionLocation = target.position;
			state = State.DistractedMove;
		}
		animationSpeed = Mathf.Lerp(animationSpeed, agent.velocity.magnitude, animationBlendSmoothing * Time.deltaTime);
		animator.SetFloat("speed", animationSpeed);
		switch (state) {
			case State.Idle:
				agent.speed = 0f;
				readyForRandom = true;
				break;
			case State.Ramble:
				agent.speed = shambleSpeed;
				if (agent.remainingDistance < 0.5f) {
					state = State.Idle;
				}
				break;
			case State.DistractedMove:
				agent.speed = shambleSpeed;
				agent.destination = distractionLocation;
				if (agent.remainingDistance < 0.5f) {
					state = State.Idle;
				}
				break;
			case State.Chase:
				if (hasLos) {
					agent.speed = moveSpeed;
					agent.destination = target.position;
				} else {
					state = State.Idle;
				}
				break;
		}


	}
}
