﻿using UnityEngine;

[RequireComponent(typeof(UISprite))]
public class PersonMarker : MonoBehaviour {

	public Survivor survivor;
	public Transform target;
	private Transform myTransform;
	private UISprite sprite;
	
	protected void Start() {
		sprite = GetComponent<UISprite>();
		target = survivor.transform;
		myTransform = transform;
	}

	protected void LateUpdate() {
		if (target == null) {
			Destroy(gameObject);
			return;
		}
		Vector3 targetScreenPos = Camera.main.WorldToScreenPoint(target.position+Vector3.up*2f);
		if (targetScreenPos.x > 32 && targetScreenPos.x < Screen.width - 32 &&
			targetScreenPos.y > 32 && targetScreenPos.y < Screen.height - 32) {
			// on screen, show speech bubble
			switch (survivor.state) {
				case Survivor.State.NeedFood:
					sprite.spriteName = "need_food";
					break;
				case Survivor.State.NeedAid:
					sprite.spriteName = "need_firstaid";
					break;
				case Survivor.State.Trapped:
					sprite.spriteName = "need_location";
					break;
				case Survivor.State.Trapped2ndFloor:
					sprite.spriteName = "need_ladder";
					break;
				case Survivor.State.Saved:
					sprite.spriteName = "saved";
					break;
			}

		} else {
			sprite.spriteName = "person_marker";
			targetScreenPos.x = Mathf.Clamp(targetScreenPos.x, 32, Screen.width - 32);
			targetScreenPos.y = Mathf.Clamp(targetScreenPos.y, 32, Screen.height - 32);
			sprite.enabled = targetScreenPos.z > 0;
		}
		
		sprite.MakePixelPerfect();
		myTransform.localPosition = targetScreenPos;
	}
}