﻿using System.Collections;
using Holoville.HOTween;
using UnityEngine;

public class UIManager : MonoBehaviour {

	#region SINGLETON

	private static UIManager _instance;

	public static UIManager Instance {
		get { return _instance; }
	}

	public void Awake() {
		if (UIManager.Instance != null) {
			DestroyImmediate(gameObject);
			return;
		}
		NoView();
		_instance = this;
	}

	public void OnApplicationQuit() {
		_instance = null;
	}

	#endregion

	public enum View {
		Title, Story, InGame, GameOver
	}

	public View currentView;
	public GameObject[] viewObjects;

	public UISprite blackout;
	private string currentSpacebarFunction;
	public UILabel spacebarLabel;
	public PersonMarker personMarkerPrefab;
	public Transform personMarkerParent;
	public UILabel gameOverSubtitle;


	public void NoView() {
		for (int i = 0; i < viewObjects.Length; i++) {
			viewObjects[i].SetActive(false);
		}
	}

	public void AddPersonMarker(Survivor survivor) {
		PersonMarker pm = (PersonMarker) Instantiate(personMarkerPrefab);
		pm.transform.parent = personMarkerParent;
		pm.transform.localScale = Vector3.one;
		pm.survivor = survivor;
	}

	public void FadeBlackout(float alpha, float time = 1f) {
		HOTween.To(blackout, time,
		           new TweenParms().Prop("color", new Color(0, 0, 0, alpha)).UpdateType(
		           	UpdateType.TimeScaleIndependentUpdate));
	}

	public void SwitchView(View newView, bool skipPreviousHide = false) {
		if (!skipPreviousHide) {
			foreach (UIWidget w in viewObjects[(int) currentView].GetComponentsInChildren<UIWidget>()) {
				HOTween.To(w, 1f,
				           new TweenParms().Prop("color", new Color(w.color.r, w.color.g, w.color.b, 0f)).UpdateType(
				           	UpdateType.TimeScaleIndependentUpdate));
			}
			StartCoroutine(DisableView(currentView));
		}
		currentView = newView;
		viewObjects[(int)newView].SetActive(true);
	}

	private IEnumerator DisableView(View view) {
		yield return new WaitForSeconds(1f * Time.timeScale);
		viewObjects[(int)view].SetActive(false);
		foreach (UIWidget w in viewObjects[(int)view].GetComponentsInChildren<UIWidget>()) {
			w.color = new Color(w.color.r, w.color.g, w.color.b, 1f);
		}
	}

	public void SpaceBarDoes(string what) {
		if (what != currentSpacebarFunction) {
			currentSpacebarFunction = what;
			spacebarLabel.text = "";
			HOTween.Kill(spacebarLabel);
			HOTween.To(spacebarLabel, 0.25f, new TweenParms().Prop("text", what).Ease(EaseType.EaseOutQuint));
		}
	}

	public void GameOver(string gameOverMessage) {
		SwitchView(View.GameOver);
		
		FadeBlackout(1f, 2f);
		gameOverSubtitle.text = gameOverMessage;
		
	}
}

